detect_reiserfs()
{
    which debugreiserfs >/dev/null 2>&1 && debugreiserfs $device >/dev/null 2>&1
}

describe_reiserfs()
{
    echo "    reiserfs"
}

scan_reiserfs()
{
    local blocksize
    local block
    local subsector

    echo "fstype reiserfs"

    # Step 6: Get filesystem blocksize and convert sector number to filesystem block number

    blocksize="`debugreiserfs $device 2>/dev/null | awk '/^Blocksize:/{print $2/512}'`"
    
    if [ -z "$blocksize" ]; then
	echo "! $device is not reiserfs" 1>&2
	exit 1
    fi

    block="`dc -e "$sector $blocksize ~ n [ ] n p"`" #`"
    subsector="${block% *}"
    block="${block#* }"

    echo "blocksize $blocksize"
    echo "block $block"
    echo "subsector $subsector"

    # Step 7: Check, whether block is in use

    if debugreiserfs -1 $block $device 2>&1 >/dev/null | grep -qF "free in ondisk bitmap"; then
	echo "blockstate free"
	exit 0
    fi
    echo "blockstate used"

    # Use debugreiserfs -1 to check the block type. This however only works if the block is readable.
    type="`debugreiserfs -1 $block $device 2>/dev/null | sed -e '/^=*$/d' | head -1`"

    case "$type" in
	"Looks like unformatted")  type="data"       	;;
	"Reiserfs super block"*)   type="superblock" 	;;
	"LEAF NODE"*)              type="meta"       	;;
	"INTERNAL NODE"*)          type="meta"       	;;
	"Desc block"*)             type="journal"    	;;
	*)                         type=""      	;;
    esac

    if [ -n "$type" ]; then
	echo "blocktype $type"
    fi

    # Step 8: Find object id to which this block belongs
    # Step 9: Find file name(s) referencing this object id

    # Currently we only look for $block in indirect blocks.

    python - $device $block <<EOF
import sys
import os
import os.path

device = sys.argv[1]
blocknr = int(sys.argv[2])

dirtree = {}
blockid = None

fp = os.popen("debugreiserfs -d %s 2>/dev/null" % device)

def parse_leafnode():
    global fp
    l = fp.readline()
    #sys.stderr.write("> leafnode(1): %s\n" % repr(l))
    if not l.startswith("LEAF NODE") : return
    for i in range(4) : fp.readline()
    while True:
        l = fp.readline()
        #sys.stderr.write("> leafnode(2): %s\n" % repr(l))
        parts = l.split("|")
        if len(parts)<2 : return
        parts = parts[2].split()
        if len(parts)<4 : return
        obid=(int(parts[0]),int(parts[1]))
        if   parts[3] == "DIR" : parse_dir(obid)
        elif parts[3] == "IND" : parse_indirect(obid)
        else:
            while True:
                l =  fp.readline().strip()
                #sys.stderr.write("> leafnode(3): %s\n" % repr(l))
                if l == 79*"-" or l == 67*"=" : break

def parse_dir(obid):
    global fp
    global dirtree
    fp.readline()
    while True:
        try:
            l = fp.readline().rstrip()
            #sys.stderr.write("> dir: %s\n" % repr(l))
            if l == 79*"-" or l == 67*"=": return
            if l.endswith("not set"): continue
            i = l.index("\"(")
            name = l[6:6+int(l[i+2:i+5])]
            if name == "." or name == ".." : continue
            i = l.index("[",len(name)+12)
            entryid = tuple(map(int,l[i+1:l.index("]",i+1)].split()))
            entry = dirtree.get(entryid)
            if entry is None : entry = dirtree[entryid] = set()
            entry.add((name,obid))
        except ValueError:
            pass

def parse_indirect(obid):
    global fp
    global blocknr
    global blockid
    fp.readline()
    for pointer in fp.readline().strip()[1:-1].split():
        i = pointer.find("(")
        if i == -1:
            blkmin = blkmax = int(pointer)
        else:
            blkmin = int(pointer[:i])
            blkmax = blkmin+int(pointer[i+1:-1])-1
        if blocknr >= blkmin and blocknr <= blkmax:
            blockid = obid
    fp.readline()

def scan_path(id):
    global dirtree
    entry = dirtree.get(id)
    #sys.stderr.write("> scan_path: %s->%s\n" % (id, entry))
    if entry is None:
        return [ '/' ]
    else:
	rv = []
	for name, parent in entry:
	    for path in scan_path(parent):
      		rv.append(os.path.join(path,name))
	return rv

def parse():
    global fp
    while True:
	while True:
	    l = fp.readline()
            #sys.stderr.write("> parse: %s\n" % repr(l))
	    if not(l) : return
            if l.strip() == 67*"=" : break
	parse_leafnode()

parse()

if blockid is None:
    sys.exit(0)

sys.stdout.write("objectid %d %d\n" % blockid)

for path in scan_path(blockid):
    if path != '/':
        sys.stdout.write("name %s\n" % path)
EOF

}

register_scanner "reiserfs"
